# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
from stix.common import EncodedCDATA
from stix.indicator import test_mechanism
import stix.bindings.extensions.test_mechanism.snort as snort_tm_binding
from mixbox import fields

@stix.register_extension
class SnortTestMechanism(test_mechanism._BaseTestMechanism):
    _namespace = "http://stix.mitre.org/extensions/TestMechanism#Snort-1"
    _binding = snort_tm_binding
    _binding_class = _binding.SnortTestMechanismType
    _XSI_TYPE = "snortTM:SnortTestMechanismType"
    
    product_name = fields.TypedField("Product_Name", EncodedCDATA)
    version = fields.TypedField("Version", EncodedCDATA)
    rules = fields.TypedField("Rule", EncodedCDATA, multiple=True, key_name="rules")
    event_filters = fields.TypedField("Event_Filter", EncodedCDATA, multiple=True, key_name="event_filters")
    rate_filters = fields.TypedField("Rate_Filter", EncodedCDATA, multiple=True, key_name="rate_filters")
    event_suppressions = fields.TypedField("Event_Suppression", EncodedCDATA, multiple=True, key_name="event_suppressions")
    
    def __init__(self, id_=None, idref=None):
        super(SnortTestMechanism, self).__init__(id_=id_, idref=idref)
    


# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# internal
import stix
import stix.utils as utils
import stix.bindings.incident as incident_binding
from stix.common.datetimewithprecision import DATETIME_PRECISION_VALUES

# relative
from .coa import COATaken

from mixbox import fields, entities

def validate_precision(instance, value):
    if value and (value not in DATETIME_PRECISION_VALUES):
        error = "time_precision must be one of {0}. Received '{1}'"
        error = error.format(DATETIME_PRECISION_VALUES, value)
        raise ValueError(error)

class JournalEntry(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = incident_binding.JournalEntryType
    
    value = fields.TypedField("valueOf_", key_name="value")
    author = fields.TypedField("author")
    time = fields.TypedField("time")     #TDOO: utils.dates.parse_value(value)
    time_precision = fields.TypedField("time_precision", preset_hook=validate_precision)
    
    def __init__(self, value=None):
        super(JournalEntry, self).__init__()
        self.value = value
        self.time_precision = 'second'


class HistoryItem(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = incident_binding.HistoryItemType
    
    action_entry = fields.TypedField("Action_Entry", COATaken)
    journal_entry = fields.TypedField("Journal_Entry", JournalEntry)
    
    def __init__(self):
        super(HistoryItem, self).__init__()


class History(stix.EntityList):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = incident_binding.HistoryType

    history_items = fields.TypedField("History_Item", HistoryItem, multiple=True, key_name="history_items")
    
    @classmethod
    def _dict_as_list(cls):
        return False
    


# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# external
from mixbox import fields
from cybox.core import Observables

# internal
import stix
from stix.common import vocabs
from stix.common.related import GenericRelationshipList, RelatedPackageRefs, RelatedCOA
from stix.common.vocabs import VocabField
from stix.common.statement import Statement
from stix.common.information_source import InformationSource
import stix.bindings.course_of_action as coa_binding

# relative
from .objective import Objective
from .structured_coa import StructuredCOAFactory, _BaseStructuredCOA

# Redefines
Stage = vocabs.COAStage
COAType = vocabs.CourseOfActionType


class RelatedCOAs(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/CourseOfAction-1"
    _binding = coa_binding
    _binding_class = coa_binding.RelatedCOAsType

    related_coa = fields.TypedField(
        name="Related_COA",
        type_=RelatedCOA,
        multiple=True,
        key_name="coas"
    )


class CourseOfAction(stix.BaseCoreComponent):
    """Implementation of the STIX Course of Action.

    Args:
        id_ (optional): An identifier. If ``None``, a value will be generated
            via ``mixbox.idgen.create_id()``. If set, this will unset the
            ``idref`` property.
        idref (optional): An identifier reference. If set this will unset the
            ``id_`` property.
        timestamp (optional): A timestamp value. Can be an instance of
            ``datetime.datetime`` or ``str``.
        description: A description of the purpose or intent of this object.
        short_description: A short description of the intent
            or purpose of this object.
        title: The title of this object.

    """
    _binding = coa_binding
    _binding_class = coa_binding.CourseOfActionType
    _namespace = "http://stix.mitre.org/CourseOfAction-1"
    _version = "1.2"
    _ALL_VERSIONS = ("1.0", "1.0.1", "1.1", "1.1.1", "1.2")
    _ID_PREFIX = 'coa'

    stage = VocabField("Stage", Stage)
    type_ = VocabField("Type", COAType)
    objective = fields.TypedField("Objective", Objective)
    parameter_observables = fields.TypedField("Parameter_Observables", Observables)
    structured_coa = fields.TypedField("Structured_COA", type_=_BaseStructuredCOA, factory=StructuredCOAFactory)
    impact = fields.TypedField("Impact", Statement)
    cost = fields.TypedField("Cost", Statement)
    efficacy = fields.TypedField("Efficacy", Statement)
    related_coas = fields.TypedField("Related_COAs", RelatedCOAs)
    related_packages = fields.TypedField("Related_Packages", RelatedPackageRefs)
    information_source = fields.TypedField("Information_Source", InformationSource)

    def __init__(self, id_=None, idref=None, timestamp=None, title=None,
                 description=None, short_description=None):

        super(CourseOfAction, self).__init__(
            id_=id_,
            idref=idref,
            timestamp=timestamp,
            title=title,
            description=description,
            short_description=short_description
        )

        self.related_coas = RelatedCOAs()
        self.related_packages = RelatedPackageRefs()

# alias for CourseOfAction
COA = CourseOfAction



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from mixbox import fields

import stix
from stix.data_marking import MarkingStructure
import stix.bindings.extensions.marking.terms_of_use_marking as tou_marking_binding


@stix.register_extension
class TermsOfUseMarkingStructure(MarkingStructure):
    _binding = tou_marking_binding
    _binding_class = tou_marking_binding.TermsOfUseMarkingStructureType
    _namespace = 'http://data-marking.mitre.org/extensions/MarkingStructure#Terms_Of_Use-1'
    _XSI_TYPE = "TOUMarking:TermsOfUseMarkingStructureType"

    terms_of_use = fields.TypedField("Terms_Of_Use")

    def __init__(self, terms_of_use=None):
        super(TermsOfUseMarkingStructure, self).__init__()
        self.terms_of_use = terms_of_use



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.incident as incident_binding
from stix.common import vocabs, VocabString
from mixbox import entities, fields

class IndirectImpactSummary(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding  = incident_binding
    _binding_class = incident_binding.IndirectImpactSummaryType

    loss_of_competitive_advantage = vocabs.VocabField("Loss_Of_Competitive_Advantage", vocabs.SecurityCompromise)
    brand_and_market_damage = vocabs.VocabField("Brand_And_Market_Damage", vocabs.SecurityCompromise)
    increased_operating_costs = vocabs.VocabField("Increased_Operating_Costs", vocabs.SecurityCompromise)
    legal_and_regulatory_costs = vocabs.VocabField("Legal_And_Regulatory_Costs", vocabs.SecurityCompromise)

    def __init__(self):
        super(IndirectImpactSummary, self).__init__()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import copy
import unittest

from mixbox.vendor.six import BytesIO

from stix.test import EntityTestCase, assert_warnings
from stix.test import report_test
from stix.test.common import kill_chains_test, related_test

from . import stix_header_test

from stix import core
from stix.core import stix_package
from stix.campaign import Campaign
from stix.coa import CourseOfAction
from stix.exploit_target import ExploitTarget
from stix.indicator import Indicator
from stix.incident import Incident
from stix.threat_actor import ThreatActor
from stix.ttp import TTP
from stix.utils import silence_warnings, now


class CampaignsTests(EntityTestCase, unittest.TestCase):
    klass = stix_package.Campaigns

    _full_dict = [
        {'idref': 'example:test-1'}
    ]


class COAsTests(EntityTestCase, unittest.TestCase):
    klass = stix_package.CoursesOfAction

    _full_dict = [
        {'idref': 'example:test-1'}
    ]

class ExploitTargetsTests(EntityTestCase, unittest.TestCase):
    klass = stix_package.ExploitTargets

    _full_dict = [
        {'idref': 'example:test-1'}
    ]

class IncidentsTests(EntityTestCase, unittest.TestCase):
    klass = stix_package.Incidents

    _full_dict = [
        {'idref': 'example:test-1'}
    ]

class IndicatorsTests(EntityTestCase, unittest.TestCase):
    klass = stix_package.Indicators

    _full_dict = [
        {'idref': 'example:test-1'}
    ]


class ThreatActorsTests(EntityTestCase, unittest.TestCase):
    klass = stix_package.ThreatActors

    _full_dict = [
        {'idref': 'example:test-1'}
    ]

class TTPsTests(EntityTestCase, unittest.TestCase):
    klass = stix_package.TTPs

    _full_dict = {
        'kill_chains': kill_chains_test.KillChainsTests._full_dict,
        'ttps': [
            {'idref': 'example:test-1'}
        ]
    }


class ReportsTests(EntityTestCase, unittest.TestCase):
    klass = stix_package.Reports

    _full_dict = [
        report_test.ReportTests._full_dict
    ]


class STIXPackageTests(EntityTestCase, unittest.TestCase):
    klass = core.STIXPackage
    _full_dict = {
        'stix_header': stix_header_test.STIXHeaderTests._full_dict,
        'campaigns': CampaignsTests._full_dict,
        'courses_of_action': COAsTests._full_dict,
        'exploit_targets': ExploitTargetsTests._full_dict,
        'incidents': IncidentsTests._full_dict,
        'indicators': IndicatorsTests._full_dict,
        'observables':  {
            'major_version': 2,
            'minor_version': 1,
            'update_version': 0,
            'observables': [
                {
                    'idref': "example:Observable-1"
                }
            ]
        },
        'threat_actors': ThreatActorsTests._full_dict,
        'ttps': TTPsTests._full_dict,
        'related_packages': related_test.RelatedPackagesTests._full_dict,
        'reports': ReportsTests._full_dict,
        'version': "1.2"
    }


    @silence_warnings
    def test_deepcopy(self):
        """Test copy.deepcopy() against parsed document.

        Previous versions of python-stix would cause an exception with
        copy.deepcopy() when applied to a parsed STIX component which contained
        timestamp information.

        This was due to the lack of a __reduce__ function defined on the
        bindings._FixedTZOffset class.

        """
        package = core.STIXPackage.from_xml(
            BytesIO(
                core.STIXPackage().to_xml()
            )
        )

        copied = copy.deepcopy(package)
        self.assertEqual(package.timestamp, copied.timestamp)

    @assert_warnings
    def test_deprecated_idref(self):
        p = core.STIXPackage()
        p.idref = "test"
        self.assertEqual(p.idref, "test")

    @assert_warnings
    def test_deprecated_timestamp(self):
        p = core.STIXPackage()
        ts = now()
        p.timestamp = ts
        self.assertEqual(ts, p.timestamp)

    @assert_warnings
    def test_campaign_idref_deprecation(self):
        package = core.STIXPackage()
        package.add(Campaign(idref='test-idref-dep'))

    @assert_warnings
    def test_coa_idref_deprecation(self):
        package = core.STIXPackage()
        package.add(CourseOfAction(idref='test-idref-dep'))

    @assert_warnings
    def test_et_idref_deprecation(self):
        package = core.STIXPackage()
        package.add(ExploitTarget(idref='test-idref-dep'))

    @assert_warnings
    def test_incident_idref_deprecation(self):
        package = core.STIXPackage()
        package.add(Incident(idref='test-idref-dep'))

    @assert_warnings
    def test_indicator_idref_deprecation(self):
        package = core.STIXPackage()
        package.add(Indicator(idref='test-idref-dep'))

    @assert_warnings
    def test_ta_idref_deprecation(self):
        package = core.STIXPackage()
        package.add(ThreatActor(idref='test-idref-dep'))

    @assert_warnings
    def test_ttp_idref_deprecation(self):
        package = core.STIXPackage()
        package.add(TTP(idref='test-idref-dep'))

    @assert_warnings
    def test_related_package_idref_deprecation(self):
        package = core.STIXPackage()
        package.add_related_package(core.STIXPackage(idref='foo'))


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from stix.test import EntityTestCase, TypedListTestCase, assert_warnings
import stix.test.data_marking_test as data_marking_test
from stix.test.common import (
    confidence_test, information_source_test, statement_test, related_test,
    activity_test
)

from stix.test import assert_warnings
from stix.core import STIXPackage
import stix.campaign as campaign


class NamesTests(EntityTestCase, unittest.TestCase):
    klass = campaign.Names

    _full_dict = {
        'names': [
            "Dancing Hippos",
            "Crazy Squirrels",
            {
                'value': "Medium",
                'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'
            }
        ]
    }


class IntendedEffectsTests(TypedListTestCase, unittest.TestCase):
    klass = campaign.Campaign

    _partial_dict = [
        statement_test.StatementTests._full_dict
    ]
    
    _full_dict = {
        'intended_effects': _partial_dict,    
    }

class RelatedTTPsTest(EntityTestCase, unittest.TestCase):
    klass = campaign.RelatedTTPs

    _full_dict = {
        'scope': 'exclusive',
        'ttps': [
            related_test.RelatedTTPTests._full_dict,
        ]
    }


class RelatedIncidentsTests(EntityTestCase, unittest.TestCase):
    klass = campaign.RelatedIncidents

    _full_dict = {
        'incidents': [
            related_test.RelatedIncidentTests._full_dict
        ]
    }


class RelatedIndicatorsTests(EntityTestCase, unittest.TestCase):
    klass = campaign.RelatedIndicators

    _full_dict = {
        'indicators': [
            related_test.RelatedIndicatorTests._full_dict
        ]
    }

class AttributionTests(EntityTestCase, unittest.TestCase):
    klass = campaign.Attribution

    _full_dict = {
        'scope': 'exclusive',
        'threat_actors': [
            related_test.RelatedThreatActorTests._full_dict,
        ]
    }


class AttributionListTests(TypedListTestCase, unittest.TestCase):
    klass = campaign.Campaign

    _full_dict = { 'attribution': [
        AttributionTests._full_dict
    ] }


class AssociatedCampaignsTests(EntityTestCase, unittest.TestCase):
    klass = campaign.AssociatedCampaigns

    _full_dict = {
        'scope': 'inclusive',
        'campaigns': [
            related_test.RelatedCampaignTests._full_dict
        ]
    }


class ActivityTests(TypedListTestCase, unittest.TestCase):
    klass = campaign.Activity

    _full_dict = activity_test.ActivityTests._full_dict
    


class CampaignTest(EntityTestCase, unittest.TestCase):
    klass = campaign.Campaign
    _full_dict = {
        'id': "example:test-1",
        'timestamp': "2014-01-31T06:14:46",
        'version': '1.2',
        'title': 'Purple Elephant',
        'description': 'A pretty novice set of actors.',
        'short_description': 'novices',
        'names': NamesTests._full_dict,
        'intended_effects': IntendedEffectsTests._partial_dict,
        'status': {
            'value': "Ongoing",
            'xsi:type':'stixVocabs:CampaignStatusVocab-1.0'
        },
        'related_ttps': RelatedTTPsTest._full_dict,
        'related_incidents': RelatedIncidentsTests._full_dict,
        'related_indicators': RelatedIndicatorsTests._full_dict,
        'attribution': [AttributionTests._full_dict],
        'associated_campaigns': AssociatedCampaignsTests._full_dict,
        'confidence': confidence_test.ConfidenceTests._full_dict,
        'activity': [ActivityTests._full_dict],
        'information_source': information_source_test.InformationSourceTests._full_dict,
        'handling': data_marking_test.MarkingTests._full_dict,
        'related_packages': related_test.RelatedPackageRefsTests._full_dict
    }

    def test_add_description(self):
        o1 = self.klass()
        o2 = self.klass()

        o1.add_description("Test")
        o2.descriptions.add("Test")

        self.assertEqual(
            o1.descriptions.to_dict(),
            o2.descriptions.to_dict()
        )

    def test_add_short_description(self):
        o1 = self.klass()
        o2 = self.klass()

        o1.add_short_description("Test")
        o2.short_descriptions.add("Test")

        self.assertEqual(
            o1.short_descriptions.to_dict(),
            o2.short_descriptions.to_dict()
        )

    @assert_warnings
    def test_deprecated_related_packages(self):
        c = campaign.Campaign()
        c.related_packages.append(STIXPackage())

    @assert_warnings
    def test_deprecated_related_indicators(self):
        from stix.indicator import Indicator

        c = campaign.Campaign()
        c.related_indicators.append(Indicator())
        self.assertEqual(1, len(c.related_indicators))


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""Tests for various encoding issues throughout the library"""

import unittest
from mixbox.vendor.six import BytesIO, text_type
from mixbox import binding_utils

from stix.core import STIXHeader, STIXPackage
from stix.campaign import Campaign
from stix.indicator import Indicator
from stix.incident import Incident
from stix.exploit_target import ExploitTarget
from stix.threat_actor import ThreatActor
from stix.ttp import TTP
from stix.common import StructuredText
from stix.incident import affected_asset
from stix.utils import silence_warnings

from stix.test import round_trip

UNICODE_STR = u"❤ ♎ ☀ ★ ☂ ♞ ☯ ☭ ☢ €☎⚑ ❄♫✂"

XML = \
u"""
<stix:STIX_Package
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:stix="http://stix.mitre.org/stix-1"
    xmlns:example="http://example.com/"
    id="example:Indicator-ba1d406e-937c-414f-9231-6e1dbe64fe8b" version="1.2" timestamp="2014-05-08T09:00:00.000000Z">
    <stix:STIX_Header>
        <stix:Title>{0}</stix:Title>
    </stix:STIX_Header>
</stix:STIX_Package>
""".format(UNICODE_STR)


class EncodingTests(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.orig_encoding = binding_utils.ExternalEncoding
        binding_utils.ExternalEncoding = 'utf-16'

    @classmethod
    def tearDownClass(cls):
        binding_utils.ExternalEncoding = cls.orig_encoding

    def _test_equal(self, obj1, obj2):
        self.assertEqual(obj1.title, obj2.title)
        self.assertEqual(obj1.description.value, obj2.description.value)
        self.assertEqual(obj1.short_description.value, obj2.short_description.value)

    def test_structured_text(self):
        s = StructuredText(UNICODE_STR)
        s2 = round_trip(s)
        self.assertEqual(s.value, s2.value)

    def test_asset_type(self):
        a = affected_asset.AssetType()
        a.count_affected = 1
        a2 = round_trip(a)
        self.assertEqual(a.count_affected, a2.count_affected)

    @silence_warnings
    def test_stix_header(self):
        header = STIXHeader()
        header.title = UNICODE_STR
        header.description = UNICODE_STR
        header.short_description = UNICODE_STR
        header2 = round_trip(header)
        self._test_equal(header, header2)

    def test_indicator(self):
        i = Indicator()
        i.title = UNICODE_STR
        i.description = UNICODE_STR
        i.short_description = UNICODE_STR
        i2 = round_trip(i)
        self._test_equal(i, i2)

    def test_incident(self):
        i = Incident()
        i.title = UNICODE_STR
        i.description = UNICODE_STR
        i.short_description = UNICODE_STR
        i2 = round_trip(i)
        self._test_equal(i, i2)

    def test_ttp(self):
        t = TTP()
        t.title = UNICODE_STR
        t.description = UNICODE_STR
        t.short_description = UNICODE_STR
        t2 = round_trip(t)
        self._test_equal(t, t2)

    def test_ta(self):
        t = ThreatActor()
        t.title = UNICODE_STR
        t.description = UNICODE_STR
        t.short_description = UNICODE_STR
        t2 = round_trip(t)
        self._test_equal(t, t2)

    def test_et(self):
        e = ExploitTarget()
        e.title = UNICODE_STR
        e.description = UNICODE_STR
        e.short_description = UNICODE_STR
        e2 = round_trip(e)
        self._test_equal(e, e2)

    def test_campaign(self):
        c = Campaign()
        c.title = UNICODE_STR
        c.description = UNICODE_STR
        c.short_description = UNICODE_STR
        c2 = round_trip(c)
        self._test_equal(c, c2)

    @silence_warnings
    def test_to_xml_utf16_encoded(self):
        encoding = 'utf-16'
        s = STIXHeader()
        s.title = UNICODE_STR
        xml = s.to_xml(encoding=encoding)
        print(xml)
        self.assertTrue(UNICODE_STR in xml.decode(encoding))

    @silence_warnings
    def test_to_xml_default_encoded(self):
        s = STIXHeader()
        s.title = UNICODE_STR
        xml = s.to_xml()
        self.assertTrue(UNICODE_STR in xml.decode('utf-8'))

    @silence_warnings
    def test_to_xml_no_encoding(self):
        s = STIXHeader()
        s.title = UNICODE_STR
        xml = s.to_xml(encoding=None)
        self.assertTrue(isinstance(xml, text_type))
        self.assertTrue(UNICODE_STR in xml)

    @silence_warnings
    def test_from_xml_utf16_encoded(self):
        utf16_xml = XML.encode('utf-16')
        sio = BytesIO(utf16_xml)
        sp = STIXPackage.from_xml(sio, encoding='utf-16')
        header = sp.stix_header
        self.assertEqual(header.title, UNICODE_STR)

    @silence_warnings
    def test_from_xml_default_encoded(self):
        utf8_xml = XML.encode('utf-8')
        sio = BytesIO(utf8_xml)
        sp = STIXPackage.from_xml(sio)
        header = sp.stix_header
        self.assertEqual(header.title, UNICODE_STR)

    @silence_warnings
    def test_utf16_roundtrip(self):
        sh = STIXHeader()
        sh.title = UNICODE_STR
        sp = STIXPackage()
        sp.stix_header = sh

        # serialize as utf-16
        xml16 = sp.to_xml(encoding='utf-16')

        # deserialize as utf-16
        sp2 = STIXPackage.from_xml(BytesIO(xml16), encoding='utf-16')
        sh2 = sp2.stix_header

        # check that the titles align
        self.assertEqual(sh.title, sh2.title)


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2016, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import lxml.etree as et

import stix
import stix.utils as utils
import stix.common as common
import stix.bindings.extensions.identity.ciq_identity_3_0 as ciq_identity_binding

from mixbox.vendor.six import string_types


XML_NS_XPIL     = "urn:oasis:names:tc:ciq:xpil:3"
XML_NS_XNL      = "urn:oasis:names:tc:ciq:xnl:3"
XML_NS_XAL      = "urn:oasis:names:tc:ciq:xal:3"
XML_NS_STIX_EXT = "http://stix.mitre.org/extensions/Identity#CIQIdentity3.0-1"

et.register_namespace('xpil', XML_NS_XPIL)
et.register_namespace('xnl', XML_NS_XNL)
et.register_namespace('xal', XML_NS_XAL)
et.register_namespace('stix-ciqidentity', XML_NS_STIX_EXT)


@stix.register_extension
class CIQIdentity3_0Instance(common.Identity):
    _binding = ciq_identity_binding
    _binding_class = _binding.CIQIdentity3_0InstanceType
    _namespace = "http://stix.mitre.org/extensions/Identity#CIQIdentity3.0-1"
    _XML_NS_PREFIX = "stix-ciqidentity"
    _XML_TYPE = "CIQIdentity3.0InstanceType"
    _XSI_TYPE = "stix-ciqidentity:CIQIdentity3.0InstanceType"

    def __init__(self, roles=None, specification=None):
        super(CIQIdentity3_0Instance, self).__init__()
        self.roles = roles
        self.specification = specification if specification else STIXCIQIdentity3_0()

    @property
    def roles(self):
        return self._roles

    @roles.setter
    def roles(self, valuelist):
        self._roles = []

        if not valuelist:
            return

        for role in valuelist:
            self.add_role(role)

    def add_role(self, role):
        if not isinstance(role, string_types):
            raise ValueError('role is not instance of string_types')

        self.roles.append(role)

    @property
    def specification(self):
        return self._specification

    @specification.setter
    def specification(self, value):
        if value and not isinstance(value, STIXCIQIdentity3_0):
            raise ValueError('value not instance of STIXCIQIdentity3_0Type')

        self._specification = value

    def to_obj(self, ns_info=None):
        obj = super(CIQIdentity3_0Instance, self).to_obj(ns_info=ns_info)
        obj.xsi_type = self._XSI_TYPE

        if self.roles:
            for role in self.roles:
                obj.add_Role(role)

        if self.specification:
            obj.Specification = self.specification.to_obj(ns_info=ns_info)

        return obj

    @classmethod
    def from_obj(cls, cls_obj):
        obj = super(CIQIdentity3_0Instance, cls).from_obj(cls_obj)

        roles = cls_obj.Role
        specification = cls_obj.Specification

        if roles:
            for role in roles:
                obj.add_role(role)

        if specification is not None:
            obj.specification = STIXCIQIdentity3_0.from_obj(specification)

        return obj

    def to_dict(self):
        d = super(CIQIdentity3_0Instance, self).to_dict()
        d['xsi:type'] = self._XSI_TYPE

        if self.roles:
            d['roles'] = [str(x) for x in self.roles]
        if self.specification:
            d['specification'] = self.specification.to_dict()
        return d

    @classmethod
    def from_dict(cls, cls_dict):
        if not cls_dict:
            return None

        obj = super(CIQIdentity3_0Instance, cls).from_dict(cls_dict)

        roles = cls_dict.get('roles', [])
        specification = cls_dict.get('specification')

        for role in roles:
            obj.add_role(role)

        if specification:
            obj.specification = STIXCIQIdentity3_0.from_dict(specification)

        return obj


class STIXCIQIdentity3_0(stix.Entity):
    _namespace = "http://stix.mitre.org/extensions/Identity#CIQIdentity3.0-1"
    XML_TAG = "{%s}Specification" % _namespace

    def __init__(self, party_name=None, languages=None, addresses=None,
                 organisation_info=None, electronic_address_identifiers=None,
                 free_text_lines=None, contact_numbers=None,
                 nationalities=None):
        self.party_name = party_name
        self.languages = languages
        self.addresses = addresses
        self.organisation_info = organisation_info
        self.electronic_address_identifiers = electronic_address_identifiers
        self.free_text_lines = free_text_lines
        self.contact_numbers = contact_numbers
        self.nationalities = nationalities

    @property
    def addresses(self):
        return self._addresses

    @addresses.setter
    def addresses(self, value):
        self._addresses = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_address(v)
        else:
            self.add_address(value)

    def add_address(self, value):
        if not value:
            return
        elif isinstance(value, Address):
            self.addresses.append(value)
        else:
            raise ValueError('value must be instance of Address')

    @property
    def languages(self):
        return self._languages

    @languages.setter
    def languages(self, value):
        self._languages = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_language(v)
        else:
            self.add_language(value)

    def add_language(self, value):
        if not value:
            return
        elif isinstance(value, Language):
            self.languages.append(value)
        else:
            self.languages.append(Language(value))

    @property
    def party_name(self):
        return self._party_name

    @party_name.setter
    def party_name(self, value):
        if not value:
            self._party_name = None
        elif isinstance(value, PartyName):
            self._party_name = value
        else:
            raise ValueError('party_name must be instance of PartyName')

    @property
    def electronic_address_identifiers(self):
        return self._electronic_address_identifiers

    @electronic_address_identifiers.setter
    def electronic_address_identifiers(self, value):
        self._electronic_address_identifiers = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_electronic_address_identifier(v)
        else:
            self.add_electronic_address_identifier(value)

    def add_electronic_address_identifier(self, value):
        if not value:
            return
        elif isinstance(value, ElectronicAddressIdentifier):
            self.electronic_address_identifiers.append(value)
        else:
            self.electronic_address_identifiers.append(ElectronicAddressIdentifier(value))

    @property
    def free_text_lines(self):
        return self._free_text_lines

    @free_text_lines.setter
    def free_text_lines(self, value):
        self._free_text_lines = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_free_text_line(v)
        else:
            self.add_free_text_line(value)

    def add_free_text_line(self, value):
        if not value:
            return
        elif isinstance(value, FreeTextLine):
            self.free_text_lines.append(value)
        else:
            self.free_text_lines.append(FreeTextLine(value))

    @property
    def contact_numbers(self):
        return self._contact_numbers

    @contact_numbers.setter
    def contact_numbers(self, value):
        self._contact_numbers = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_contact_number(v)
        else:
            self.add_contact_number(value)

    def add_contact_number(self, value):
        if not value:
            return
        elif isinstance(value, ContactNumber):
            self.contact_numbers.append(value)
        else:
            self.contact_numbers.append(ContactNumber(value))

    @property
    def nationalities(self):
        return self._nationalities

    @nationalities.setter
    def nationalities(self, value):
        self._nationalities = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_nationality(v)
        else:
            self.add_nationality(value)

    def add_nationality(self, value):
        if not value:
            return
        elif isinstance(value, Country):
            self.nationalities.append(value)
        else:
            self.nationalities.append(Country(value))

    @property
    def organisation_info(self):
        return self._organisation_info

    @organisation_info.setter
    def organisation_info(self, value):
        if not value:
            self._organisation_info = None
        elif isinstance(value, OrganisationInfo):
            self._organisation_info = value
        else:
            raise ValueError('organisation_info must be instance of OrganisationInfo')

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None
        if not return_obj:
            return_obj = cls()

        party_name = obj.findall(PartyName.XML_TAG)
        if party_name is not None and len(party_name) > 0:
            return_obj.party_name = PartyName.from_obj(party_name[0])

        languages = obj.findall("{%s}Languages" % XML_NS_XPIL)
        if languages is not None and len(languages) > 0:
            return_obj.languages = [Language.from_obj(x) for x in languages[0]]

        addresses = obj.findall("{%s}Addresses" % XML_NS_XPIL)
        if addresses is not None and len(addresses) > 0:
            return_obj.addresses = [Address.from_obj(x) for x in addresses[0]]

        nationalities = obj.findall("{%s}Nationalities" % XML_NS_XPIL)
        if nationalities is not None and len(nationalities) > 0:
            return_obj.nationalities = [Country.from_obj(x) for x in nationalities[0]]

        organisation_info = obj.findall(OrganisationInfo.XML_TAG)
        if organisation_info is not None and len(organisation_info) > 0:
            return_obj.organisation_info = OrganisationInfo.from_obj(organisation_info[0])

        electronic_address_identifiers = obj.findall("{%s}ElectronicAddressIdentifiers" % XML_NS_XPIL)
        if electronic_address_identifiers is not None and len(electronic_address_identifiers) > 0:
            return_obj.electronic_address_identifiers = [ElectronicAddressIdentifier.from_obj(x) for x in electronic_address_identifiers[0]]

        free_text_lines = obj.findall("{%s}FreeTextLines" % XML_NS_XPIL)
        if free_text_lines is not None and len(free_text_lines) > 0:
            return_obj.free_text_lines = [FreeTextLine.from_obj(x) for x in free_text_lines[0]]

        contact_numbers = obj.findall("{%s}ContactNumbers" % XML_NS_XPIL)
        if contact_numbers is not None and len(contact_numbers) > 0:
            return_obj.contact_numbers = [ContactNumber.from_obj(x) for x in contact_numbers[0]]

        return return_obj

    def to_obj(self, return_obj=None, ns_info=None):
        # Throw away return value; this class has no _binding_class, so
        # it will return None anyway.  This to_obj() is anomalous in that it
        # returns an etree Element instead of a generateDS object.  Bindings
        # have all been hacked up to make this work.  The super call does,
        # however, do namespace collection (if ns_info is given), so it's
        # still important.
        super(STIXCIQIdentity3_0, self).to_obj(ns_info=ns_info)

        if not return_obj:
            root_tag = STIXCIQIdentity3_0.XML_TAG
            return_obj = et.Element(root_tag)

        if self.free_text_lines:
            ftl_root = et.Element("{%s}FreeTextLines" % XML_NS_XPIL)
            return_obj.append(ftl_root)
            for ftl in self.free_text_lines:
                ftl_root.append(ftl.to_obj(ns_info=ns_info))

        if self.party_name:
            return_obj.append(self.party_name.to_obj(ns_info=ns_info))

        if self.addresses:
            addresses_root = et.Element("{%s}Addresses" % XML_NS_XPIL)
            return_obj.append(addresses_root)
            for address in self.addresses:
                addresses_root.append(address.to_obj(ns_info=ns_info))

        if self.contact_numbers:
            contact_numbers_root = et.Element("{%s}ContactNumbers" % XML_NS_XPIL)
            return_obj.append(contact_numbers_root)
            for contact_number in self.contact_numbers:
                contact_numbers_root.append(contact_number.to_obj(ns_info=ns_info))

        if self.electronic_address_identifiers:
            eai_root = et.Element("{%s}ElectronicAddressIdentifiers" % XML_NS_XPIL)
            return_obj.append(eai_root)
            for eai in self.electronic_address_identifiers:
                eai_root.append(eai.to_obj(ns_info=ns_info))

        if self.organisation_info:
            return_obj.append(self.organisation_info.to_obj(ns_info=ns_info))

        if self.languages:
            languages_root = et.Element("{%s}Languages" % XML_NS_XPIL)
            return_obj.append(languages_root)
            for language in self.languages:
                languages_root.append(language.to_obj(ns_info=ns_info))

        if self.nationalities:
            nationalities_root = et.Element("{%s}Nationalities" % XML_NS_XPIL)
            return_obj.append(nationalities_root)
            for country in self.nationalities:
                country_obj = country.to_obj(ns_info=ns_info)
                country_obj.tag = "{%s}Country" % XML_NS_XPIL
                nationalities_root.append(country_obj)

        return return_obj

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.party_name = PartyName.from_dict(dict_repr.get('party_name'))
        return_obj.languages = [Language.from_dict(x) for x in dict_repr.get('languages', [])]
        return_obj.addresses = [Address.from_dict(x) for x in dict_repr.get('addresses', [])]
        return_obj.electronic_address_identifiers = [ElectronicAddressIdentifier.from_dict(x) for x in dict_repr.get('electronic_address_identifiers', [])]
        return_obj.free_text_lines = [FreeTextLine.from_dict(x) for x in dict_repr.get('free_text_lines', [])]
        return_obj.contact_numbers = [ContactNumber.from_dict(x) for x in dict_repr.get('contact_numbers', [])]
        return_obj.nationalities = [Country.from_dict(x) for x in dict_repr.get('nationalities', [])]
        return_obj.organisation_info = OrganisationInfo.from_dict(dict_repr.get('organisation_info'))

        return return_obj

    def to_dict(self):
        d = {}

        if self.party_name:
            d['party_name'] = self.party_name.to_dict()
        if self.languages:
            d['languages'] = [x.to_dict() for x in self.languages]
        if self.addresses:
            d['addresses'] = [x.to_dict() for x in self.addresses]
        if self.electronic_address_identifiers:
            d['electronic_address_identifiers'] = [x.to_dict() for x in self.electronic_address_identifiers]
        if self.free_text_lines:
            d['free_text_lines'] = [x.to_dict() for x in self.free_text_lines]
        if self.contact_numbers:
            d['contact_numbers'] = [x.to_dict() for x in self.contact_numbers]
        if self.nationalities:
            d['nationalities'] = [x.to_dict() for x in self.nationalities]
        if self.organisation_info:
            d['organisation_info'] = self.organisation_info.to_dict()

        return d


class Address(stix.Entity):
    _namespace = XML_NS_XPIL
    XML_TAG = "{%s}Address" % _namespace

    def __init__(self, free_text_address=None, country=None,
                 administrative_area=None):
        self.free_text_address = free_text_address
        self.country = country
        self.administrative_area = administrative_area

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._set_var(Country, country=value)

    @property
    def administrative_area(self):
        return self._administrative_area

    @administrative_area.setter
    def administrative_area(self, value):
        self._set_var(AdministrativeArea, administrative_area=value)

    @property
    def free_text_address(self):
        return self._free_text_address

    @free_text_address.setter
    def free_text_address(self, value):
        self._set_var(FreeTextAddress, free_text_address=value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(Address, self).to_obj(ns_info=ns_info)

        if not return_obj:
            return_obj = et.Element(self.XML_TAG)

        if self.free_text_address:
            return_obj.append(self.free_text_address.to_obj(ns_info=ns_info))
        if self.country:
            return_obj.append(self.country.to_obj(ns_info=ns_info))
        if self.administrative_area:
            return_obj.append(self.administrative_area.to_obj(ns_info=ns_info))

        return return_obj

    def to_dict(self):
        d = {}
        if self.free_text_address:
            d['free_text_address'] = self.free_text_address.to_dict()
        if self.country:
            d['country'] = self.country.to_dict()
        if self.administrative_area:
            d['administrative_area'] = self.administrative_area.to_dict()
        return d

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None
        if not return_obj:
            return_obj = cls()

        free_text_address = obj.findall("{%s}FreeTextAddress" % XML_NS_XAL)
        if len(free_text_address) > 0:
            return_obj.free_text_address = FreeTextAddress.from_obj(free_text_address[0])

        country = obj.findall("{%s}Country" % XML_NS_XAL)
        if len(country) > 0:
            return_obj.country = Country.from_obj(country[0])

        administrative_area = obj.findall("{%s}AdministrativeArea" % XML_NS_XAL)
        if len(administrative_area) > 0:
            return_obj.administrative_area = AdministrativeArea.from_obj(administrative_area[0])

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.free_text_address = FreeTextAddress.from_dict(d.get('free_text_address'))
        return_obj.country = Country.from_dict(d.get('country'))
        return_obj.administrative_area = AdministrativeArea.from_dict(d.get('administrative_area'))
        return return_obj


class AdministrativeArea(stix.Entity):
    _namespace = XML_NS_XAL
    XML_TAG = "{%s}AdministrativeArea" % _namespace

    def __init__(self, name_elements=None):
        self.name_elements = name_elements

    @property
    def name_elements(self):
        return self._name_elements

    @name_elements.setter
    def name_elements(self, value):
        self._name_elements = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_name_element(v)
        else:
            self.add_name_element(value)

    def add_name_element(self, value):
        if not value:
            return
        elif isinstance(value, NameElement):
            self.name_elements.append(value)
        else:
            self.name_elements.append(NameElement(value))

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None
        if not return_obj:
            return_obj = cls()

        name_elements = obj.findall(NameElement.XML_TAG)
        if name_elements:
            for name_element in name_elements:
                return_obj.name_elements.append(NameElement.from_obj(name_element))

        return return_obj

    def to_obj(self, return_obj=None, ns_info=None):
        super(AdministrativeArea, self).to_obj(ns_info=ns_info)

        if not return_obj:
            return_obj = et.Element(self.XML_TAG)

        for name_element in self.name_elements:
            return_obj.append(name_element.to_obj(ns_info=ns_info))

        return return_obj

    def to_dict(self):
        d = {}
        if self.name_elements:
            d['name_elements'] = [x.to_dict() for x in self.name_elements]
        return d

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.name_elements = [NameElement.from_dict(x) for x in d.get('name_elements', [])]
        return return_obj


class Country(stix.Entity):
    _namespace = XML_NS_XAL
    XML_TAG = "{%s}Country" % _namespace

    def __init__(self, name_elements=None):
        self.name_elements = name_elements

    @property
    def name_elements(self):
        return self._name_elements

    @name_elements.setter
    def name_elements(self, value):
        self._name_elements = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_name_element(v)
        else:
            self.add_name_element(value)

    def add_name_element(self, value):
        if not value:
            return
        elif isinstance(value, NameElement):
            self.name_elements.append(value)
        else:
            self.name_elements.append(NameElement(value))

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None
        if not return_obj:
            return_obj = cls()

        name_elements = obj.findall("{%s}NameElement" % XML_NS_XAL)
        if name_elements:
            for name_element in name_elements:
                return_obj.name_elements.append(NameElement.from_obj(name_element))

        return return_obj

    def to_obj(self, return_obj=None, ns_info=None):
        super(Country, self).to_obj(ns_info=ns_info)

        if not return_obj:
            return_obj = et.Element(self.XML_TAG)

        for name_element in self.name_elements:
            return_obj.append(name_element.to_obj(ns_info=ns_info))

        return return_obj

    def to_dict(self):
        d = {}
        if self.name_elements:
            d['name_elements'] = [x.to_dict() for x in self.name_elements]
        return d
    
    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.name_elements = [NameElement.from_dict(x) for x in d.get('name_elements', [])]
        return return_obj


class NameElement(stix.Entity):
    _namespace = XML_NS_XAL
    XML_TAG = "{%s}NameElement" % XML_NS_XAL

    def __init__(self, value=None, name_type=None, name_code=None,
                 name_code_type=None):
        self.value = value
        self.name_type = name_type
        self.name_code = name_code
        self.name_code_type = name_code_type

    def to_obj(self, return_obj=None, ns_info=None):
        super(NameElement, self).to_obj(ns_info=ns_info)

        return_obj = et.Element(self.XML_TAG)
        return_obj.text = self.value

        if self.name_type:
            return_obj.set('{%s}NameType' % self._namespace, self.name_type)
        if self.name_code:
            return_obj.set('{%s}NameCode' % self._namespace, self.name_code)
        if self.name_code_type:
            return_obj.set('{%s}NameCodeType' % self._namespace, self.name_code_type)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.value = obj.text
        return_obj.name_type = obj.get('{%s}NameType' % cls._namespace)
        return_obj.name_code = obj.get('{%s}NameCode' % cls._namespace)
        return_obj.name_code_type = obj.get('{%s}NameCodeType' % cls._namespace)

        return return_obj

    def to_dict(self):
        d = {}
        if self.value:
            d['value'] = self.value
        if self.name_type:
            d['name_type'] = self.name_type
        if self.name_code:
            d['name_code'] = self.name_code
        if self.name_code_type:
            d['name_code_type'] = self.name_code_type
        return d

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.value = d.get('value')
        return_obj.name_type = d.get('name_type')
        return_obj.name_code = d.get('name_code')
        return_obj.name_code_type = d.get('name_code_type')
        return return_obj


class FreeTextAddress(stix.Entity):
    _namespace = XML_NS_XAL
    XML_TAG = "{%s}FreeTextAddress" % XML_NS_XAL

    def __init__(self, address_lines=None):
        self.address_lines = address_lines

    @property
    def address_lines(self):
        return self._address_lines

    @address_lines.setter
    def address_lines(self, value):
        self._address_lines = []

        if value is None or len(value) == 0:
            return
        elif utils.is_sequence(value):
            self._address_lines = value
        else:
            self._address_lines.append(value)

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None or len(obj) == 0:
            return None
        if not return_obj:
            return_obj = cls()

        address_line_tag = "{%s}AddressLine" % XML_NS_XAL
        address_lines = obj.findall(address_line_tag)
        if address_lines:
            for address_line in address_lines:
                return_obj.address_lines.append(address_line.text)

        return return_obj

    def to_obj(self, return_obj=None, ns_info=None):
        super(FreeTextAddress, self).to_obj(ns_info=ns_info)

        if not return_obj:
            return_obj = et.Element(self.XML_TAG)

        for address in self.address_lines:
            address_line = et.Element("{%s}AddressLine" % XML_NS_XAL)
            address_line.text = address
            return_obj.append(address_line)

        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.address_lines = d.get('address_lines', [])
        return return_obj

    def to_dict(self):
        d = {}
        if self.address_lines:
            d['address_lines'] = self.address_lines
        return d


class PartyName(stix.Entity):
    _namespace = XML_NS_XPIL
    XML_TAG = "{%s}PartyName" % _namespace

    def __init__(self, name_lines=None, person_names=None, organisation_names=None):
        self.name_lines = []
        self.person_names = []
        self.organisation_names = []

        if name_lines:
            for value in name_lines:
                self.add_name_line(value)

        if person_names:
            for value in person_names:
                self.add_person_name(value)

        if organisation_names:
            for value in organisation_names:
                self.add_organisation_name(value)

    def add_name_line(self, value):
        if isinstance(value, string_types):
            self.name_lines.append(NameLine(value))
        elif isinstance(value, NameLine):
            self.name_lines.append(value)
        else:
            raise ValueError('value must be a string_types or NameLine instance')

    def add_person_name(self, value):
        if isinstance(value, string_types):
            self.person_names.append(PersonName(name_elements=[value]))
        elif isinstance(value, PersonName):
            self.person_names.append(value) 
        else:
            raise ValueError('value must be instance of PersonName or string_types')

    def add_organisation_name(self, value):
        if isinstance(value, string_types):
            self.organisation_names.append(OrganisationName(name_elements=[value]))  
        elif isinstance(value, OrganisationName):
            self.organisation_names.append(value)
        else:
            raise ValueError('value must be instance of OrganisationName')

    def to_obj(self, return_obj=None, ns_info=None):
        super(PartyName, self).to_obj(ns_info=ns_info)

        if not return_obj:
            root_tag = PartyName.XML_TAG
            return_obj = et.Element(root_tag)

        for name_line in self.name_lines:
            return_obj.append(name_line.to_obj(ns_info=ns_info))

        for person_name in self.person_names:
            return_obj.append(person_name.to_obj(ns_info=ns_info))

        for organisation_name in self.organisation_names:
            return_obj.append(organisation_name.to_obj(ns_info=ns_info))

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        name_lines = obj.findall(NameLine.XML_TAG)
        if name_lines:
            for name_line_obj in name_lines:
                name_line = NameLine.from_obj(name_line_obj)
                return_obj.add_name_line(name_line)

        person_names = obj.findall(PersonName.XML_TAG)
        if person_names:
            for person_name_obj in person_names:
                person_name = PersonName.from_obj(person_name_obj)
                return_obj.add_person_name(person_name)

        org_names = obj.findall(OrganisationName.XML_TAG)
        if org_names:
            for organisation_name_obj in org_names:
                org_name = OrganisationName.from_obj(organisation_name_obj)
                return_obj.add_organisation_name(org_name)

        return return_obj

    def to_dict(self):
        d = {}

        if self.name_lines:
            for name_line in self.name_lines:
                d.setdefault('name_lines', []).append(name_line.to_dict())

        if self.organisation_names:
            for on in self.organisation_names:
                d.setdefault('organisation_names', []).append(on.to_dict())

        if self.person_names:
            for pn in self.person_names:
                d.setdefault('person_names', []).append(pn.to_dict())

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        nl_dicts = dict_repr.get('name_lines', [])
        on_dicts = dict_repr.get('organisation_names', [])
        pn_dicts = dict_repr.get('person_names', [])

        for nl in nl_dicts:
            name_line = NameLine.from_dict(nl)
            return_obj.add_name_line(name_line)

        for on in on_dicts:
            organisation_name = OrganisationName.from_dict(on)
            return_obj.add_organisation_name(organisation_name)

        for pn in pn_dicts:
            person_name = PersonName.from_dict(pn)
            return_obj.add_person_name(person_name)

        return return_obj


class NameLine(stix.Entity):
    _namespace = XML_NS_XNL
    XML_TAG = "{%s}NameLine" % _namespace

    def __init__(self, value=None, type_=None):
        self.value = value
        self.type = type_

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        if value and not isinstance(value, string_types):
            raise ValueError('value must be instance of string_types')

        self._value = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(NameLine, self).to_obj(ns_info=ns_info)

        if not return_obj:
            root_tag = NameLine.XML_TAG
            return_obj = et.Element(root_tag)

        if self.type:
            return_obj.attrib['Type'] = self.type

        if self.value:
            return_obj.text = self.value

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.value = obj.text
        return_obj.type = obj.get('Type')

        return return_obj

    def to_dict(self):
        d = {'value': self.value}

        if self.type:
            d['type'] = self.type

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.value = dict_repr.get('value', None)
        return_obj.type = dict_repr.get('type', None)

        return return_obj


class PersonName(stix.Entity):
    _namespace = XML_NS_XNL
    XML_TAG = "{%s}PersonName" % _namespace

    def __init__(self, name_elements=None):
        self.name_elements = []

        if name_elements:
            for name_element in name_elements:
                self.add_name_element(name_element)

    def add_name_element(self, value):
        if isinstance(value, string_types):
            self.name_elements.append(PersonNameElement(value=value))
        elif isinstance(value, PersonNameElement):
            self.name_elements.append(value)
        else:
            raise ValueError('value must be instance of PersonNameElement')

    def to_obj(self, return_obj=None, ns_info=None):
        super(PersonName, self).to_obj(ns_info=ns_info)

        if not return_obj:
            root_tag = PersonName.XML_TAG
            return_obj = et.Element(root_tag)

        for name_element in self.name_elements:
            return_obj.append(name_element.to_obj(ns_info=ns_info))

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        name_elements = obj.findall(PersonNameElement.XML_TAG)
        if name_elements:
            for name_element_obj in name_elements:
                person_name_element = PersonNameElement.from_obj(name_element_obj)
                return_obj.add_name_element(person_name_element)

        return return_obj

    def to_dict(self):
        d = {}

        if self.name_elements:
            for ne in self.name_elements:
                d.setdefault('name_elements', []).append(ne.to_dict())

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        ne_dicts = dict_repr.get('name_elements', [])

        for ne_dict in ne_dicts:
            return_obj.add_name_element(PersonNameElement.from_dict(ne_dict))

        return return_obj


class OrganisationName(stix.Entity):
    _namespace = XML_NS_XNL
    XML_TAG = "{%s}OrganisationName" % _namespace

    def __init__(self, name_elements=None, subdivision_names=None, type_=None):
        self.type_ = type_
        self.name_elements = name_elements
        self.subdivision_names = subdivision_names

    @property
    def name_elements(self):
        return self._name_elements

    @name_elements.setter
    def name_elements(self, value):
        self._name_elements = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_organisation_name_element(v)
        else:
            self.add_organisation_name_element(value)

    def add_organisation_name_element(self, value):
        if isinstance(value, string_types):
            self.name_elements.append(OrganisationNameElement(value=value))
        elif isinstance(value, OrganisationNameElement):
            self.name_elements.append(value)
        else:
            raise ValueError('value must be instance of OrganisationNameElement')

    @property
    def subdivision_names(self):
        return self._subdivision_names

    @subdivision_names.setter
    def subdivision_names(self, value):
        self._subdivision_names = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_subdivision_name(v)
        else:
            self.add_subdivision_name(value)

    def add_subdivision_name(self, value):
        if not isinstance(value, SubDivisionName):
            raise ValueError('value must be instance of SubDivisionName')

        self.subdivision_names.append(value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(OrganisationName, self).to_obj(ns_info=ns_info)

        if not return_obj:
            root_tag = OrganisationName.XML_TAG
            return_obj = et.Element(root_tag)

        if self.type_:
            return_obj.attrib['{%s}Type' % XML_NS_XNL] = self.type_
        for name_element in self.name_elements:
            return_obj.append(name_element.to_obj(ns_info=ns_info))
        for subdivision_name in self.subdivision_names:
            return_obj.append(subdivision_name.to_obj(ns_info=ns_info))

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.type_ = obj.attrib.get('{%s}Type' % XML_NS_XNL)

        name_elements = obj.findall(OrganisationNameElement.XML_TAG)
        if name_elements:
            for name_element_obj in name_elements:
                name_element = OrganisationNameElement.from_obj(name_element_obj)
                return_obj.add_organisation_name_element(name_element)

        sub_division_names = obj.findall(SubDivisionName.XML_TAG)
        if sub_division_names:
            for sub_division_name_obj in sub_division_names:
                sub_division_name = SubDivisionName.from_obj(sub_division_name_obj)
                return_obj.add_subdivision_name(sub_division_name)

        return return_obj

    def to_dict(self):
        d = {}

        if self.type_:
            d['type'] = self.type_
        if self.name_elements:
            for ne in self.name_elements:
                d.setdefault('name_elements', []).append(ne.to_dict())
        if self.subdivision_names:
            for sn in self.subdivision_names:
                d.setdefault('subdivision_names', []).append(sn.to_dict())

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        ne_dicts = dict_repr.get('name_elements', [])
        sn_dicts = dict_repr.get('subdivision_names', [])

        return_obj.type_ = dict_repr.get('type')
        for ne_dict in ne_dicts:
            return_obj.add_organisation_name_element(OrganisationNameElement.from_dict(ne_dict))
        for sn_dict in sn_dicts:
            return_obj.add_subdivision_name(SubDivisionName.from_dict(sn_dict))

        return return_obj


class _BaseNameElement(stix.Entity):
    """Do not instantiate directly: use PersonNameElement or
    OrganisationNameElement

    """
    def __init__(self, value=None):
        self.value = value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        # if not value:
        #    raise ValueError('value cannot be None')

        self._value = value

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not return_obj:
            raise ValueError("Must supply return_obj")

        return_obj.value = obj.valueOf_
        return return_obj

    def to_obj(self, return_obj=None, ns_info=None):
        super(_BaseNameElement, self).to_obj(ns_info=ns_info)

        return_obj.text = self.value
        return return_obj

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not return_obj:
            raise ValueError("Must supply return_obj")

        return_obj.value = dict_repr.get('value', None)
        return return_obj

    def to_dict(self):
        return dict(value=self.value)


class PersonNameElement(_BaseNameElement):
    _namespace = XML_NS_XNL
    XML_TAG = "{%s}NameElement" % _namespace

    TYPE_TITLE = 'Title'
    TYPE_PRECEDING_TITLE = 'PrecedingTitle'
    TYPE_FIRST_NAME = 'FirstName'
    TYPE_MIDDLE_NAME = 'MiddleName'
    TYPE_LAST_NAME = 'LastName'
    TYPE_OTHER_NAME = 'OtherName'
    TYPE_ALIAS = 'Alias'
    TYPE_GENERATION_IDENTIFIER = 'GenerationIdentifier'
    TYPE_DEGREE = 'Degree'

    TYPES = (
        TYPE_TITLE, TYPE_PRECEDING_TITLE, TYPE_FIRST_NAME, TYPE_LAST_NAME,
        TYPE_MIDDLE_NAME, TYPE_OTHER_NAME, TYPE_ALIAS,
        TYPE_GENERATION_IDENTIFIER, TYPE_DEGREE
    )

    def __init__(self, value=None, element_type=None):
        super(PersonNameElement, self).__init__(value)
        self.element_type = element_type

    @property
    def element_type(self):
        return self._element_type

    @element_type.setter
    def element_type(self, value):
        if value and value not in self.TYPES:
            raise ValueError('value must be one of %s: ' % (self.TYPES,))

        self._element_type = value

    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            root_tag = PersonNameElement.XML_TAG
            return_obj = et.Element(root_tag)

        super(PersonNameElement, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if self.element_type:
            return_obj.attrib['ElementType'] = self.element_type

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.element_type = obj.get('ElementType')
        return_obj.value = obj.text

        return return_obj

    def to_dict(self):
        d = {'value': self.value}

        if self.element_type:
            d['element_type'] = self.element_type

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.value = dict_repr.get('value', None)
        return_obj.element_type = dict_repr.get('element_type', None)

        return return_obj


class OrganisationNameElement(_BaseNameElement):
    _namespace = XML_NS_XNL
    XML_TAG = "{%s}NameElement" % _namespace

    TYPE_NAME_ONLY = "NameOnly"
    TYPE_TYPE_ONLY = "TypeOnly"
    TYPE_FULL_NAME = "FullName"

    TYPES = (TYPE_NAME_ONLY, TYPE_TYPE_ONLY, TYPE_FULL_NAME)

    def __init__(self, value=None, element_type=None):
        super(OrganisationNameElement, self).__init__(value)
        self.value = value
        self.element_type = element_type

    @property
    def element_type(self):
        return self._element_type

    @element_type.setter
    def element_type(self, value):
        if value and value not in self.TYPES:
            raise ValueError('value must be one of: %s ' % (self.TYPES,))

        self._element_type = value

    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            root_tag = OrganisationNameElement.XML_TAG
            return_obj = et.Element(root_tag)

        super(OrganisationNameElement, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if self.element_type:
            return_obj.attrib['{%s}ElementType' % XML_NS_XNL] = self.element_type

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.element_type = obj.get('{%s}ElementType' % XML_NS_XNL)
        return_obj.value = obj.text

        return return_obj

    def to_dict(self):
        d = {}
        if self.element_type:
            d['element_type'] = self.element_type
        if self.value:
            d['value'] = self.value

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.value = dict_repr.get('value')
        return_obj.element_type = dict_repr.get('element_type')

        return return_obj


class SubDivisionName(stix.Entity):
    _namespace = XML_NS_XNL
    XML_TAG = "{%s}SubDivisionName" % _namespace

    TYPE_DEPARTMENT = 'Department'
    TYPE_DIVISION = 'Division'
    TYPE_BRANCH = 'Branch'
    TYPE_BUSINESS_UNIT = 'BusinessUnit'
    TYPE_SCHOOL = 'School'
    TYPE_SECTION = 'Section'

    TYPES = (
        TYPE_DEPARTMENT, TYPE_DIVISION, TYPE_BRANCH, TYPE_BUSINESS_UNIT,
        TYPE_SCHOOL, TYPE_SECTION
    )

    def __init__(self, value=None, type_=None):
        self.value = value
        self.type = type_

    @property
    def type(self):
        return self._type

    @type.setter
    def type(self, value):
        if value and value not in self.TYPES:
            raise ValueError('value must be one of: %s' % (self.TYPES,))

        self._type = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(SubDivisionName, self).to_obj(ns_info=ns_info)

        if not return_obj:
            root_tag = SubDivisionName.XML_TAG
            return_obj = et.Element(root_tag)

        if self.type:
            return_obj.attrib['{%s}Type' % XML_NS_XNL] = self.type

        return_obj.text = self.value
        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.type = obj.get('{%s}Type' % XML_NS_XNL)
        return_obj.value = obj.text

        return return_obj

    def to_dict(self):
        d = {'value': self.value}

        if self.type:
            d['type'] = self.type

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.value = dict_repr.get('value')
        return_obj.type = dict_repr.get('type')
        return return_obj


class Language(stix.Entity):
    _namespace = XML_NS_XPIL
    XML_TAG = "{%s}Language" % _namespace

    def __init__(self, value=None):
        self.value = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(Language, self).to_obj(ns_info=ns_info)

        return_obj = et.Element(self.XML_TAG)
        return_obj.text = self.value
        return return_obj

    @classmethod
    def from_obj(cls, obj):
        if obj is None:
            return None

        return_obj = cls()
        return_obj.value = obj.text
        return return_obj

    def to_dict(self):
        d = {}
        if self.value:
            d['value'] = self.value
        return d

    @classmethod
    def from_dict(cls, d):
        if not d:
            return None

        return_obj = cls()
        return_obj.value = d.get('value')
        return return_obj


class ElectronicAddressIdentifier(stix.Entity):
    _namespace = XML_NS_XPIL
    XML_TAG = "{%s}ElectronicAddressIdentifier" % _namespace

    def __init__(self, value=None, type_=None):
        self.type_ = type_
        self.value = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(ElectronicAddressIdentifier, self).to_obj(ns_info=ns_info)

        return_obj = et.Element(self.XML_TAG)
        return_obj.text = self.value

        if self.type_:
            return_obj.attrib['{%s}Type' % XML_NS_XPIL] = self.type_

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.type_ = obj.attrib.get('{%s}Type' % XML_NS_XPIL)
        return_obj.value = obj.text
        return return_obj

    def to_dict(self):
        d = {}
        if self.value:
            d['value'] = self.value
        if self.type_:
            d['type'] = self.type_
        return d

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.value = d.get('value')
        return_obj.type_ = d.get('type')
        return return_obj


class OrganisationInfo(stix.Entity):
    _namespace = XML_NS_XPIL
    XML_TAG = "{%s}OrganisationInfo" % _namespace

    def __init__(self, industry_type=None):
        self.industry_type = industry_type

    def to_obj(self, return_obj=None, ns_info=None):
        super(OrganisationInfo, self).to_obj(ns_info=ns_info)

        return_obj = et.Element(self.XML_TAG)
        if self.industry_type:
            return_obj.attrib['{%s}IndustryType' % self._namespace] = self.industry_type

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.industry_type = obj.get('{%s}IndustryType' % cls._namespace)
        return return_obj

    def to_dict(self):
        d = {}
        if self.industry_type:
            d['industry_type'] = self.industry_type
        return d

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.industry_type = d.get('industry_type')
        return return_obj


class FreeTextLine(stix.Entity):
    _namespace = XML_NS_XPIL
    XML_TAG = "{%s}FreeTextLine" % _namespace

    def __init__(self, value=None, type_=None):
        self.value = value
        self.type_ = type_

    def to_obj(self, return_obj=None, ns_info=None):
        super(FreeTextLine, self).to_obj(ns_info=ns_info)

        return_obj = et.Element(self.XML_TAG)
        if self.type_:
            return_obj.attrib['{%s}Type' % self._namespace] = self.type_
        if self.value:
            return_obj.text = self.value

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.type_ = obj.get('{%s}Type' % cls._namespace)
        return_obj.value = obj.text
        return return_obj

    def to_dict(self):
        d = {}
        if self.type_:
            d['type'] = self.type_
        if self.value:
            d['value'] = self.value
        return d

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.type_ = d.get('type')
        return_obj.value = d.get('value')
        return return_obj


class ContactNumber(stix.Entity):
    _namespace = XML_NS_XPIL
    XML_TAG = "{%s}ContactNumber" % _namespace

    COM_MEDIA_TYPE_CELLPHONE = "Cellphone"
    COM_MEDIA_TYPE_FAX = "Fax"
    COM_MEDIA_TYPE_PAGER = "Pager"
    COM_MEDIA_TYPE_TELEPHONE = "Telephone"
    COM_MEDIA_TYPE_VOIP = "VOIP"

    ALLOWED_COM_MEDIA_TYPES = (
        COM_MEDIA_TYPE_CELLPHONE, COM_MEDIA_TYPE_FAX, COM_MEDIA_TYPE_PAGER,
        COM_MEDIA_TYPE_TELEPHONE, COM_MEDIA_TYPE_VOIP
    )

    def __init__(self, contact_number_elements=None, communication_media_type=None):
        self.communication_media_type = communication_media_type
        self.contact_number_elements = contact_number_elements

    @property
    def contact_number_elements(self):
        return self._contact_number_elements

    @contact_number_elements.setter
    def contact_number_elements(self, value):
        self._contact_number_elements = []
        if not value:
            return
        elif utils.is_sequence(value):
            for v in value:
                self.add_contact_number_element(v)
        else:
            self.add_contact_number_element(value)

    def add_contact_number_element(self, value):
        if not value:
            return
        elif isinstance(value, ContactNumberElement):
            self.contact_number_elements.append(value)
        else:
            self.contact_number_elements.append(ContactNumberElement(value))

    @property
    def communication_media_type(self):
        return self._communication_media_type

    @communication_media_type.setter
    def communication_media_type(self, value):
        if not value:
            self._communication_media_type = None
        elif value not in self.ALLOWED_COM_MEDIA_TYPES:
            raise ValueError('value must be one of %s' % (self.ALLOWED_COM_MEDIA_TYPES,))
        else:
            self._communication_media_type = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(ContactNumber, self).to_obj(ns_info=ns_info)
        return_obj = et.Element(self.XML_TAG)
        if self.communication_media_type:
            return_obj.attrib['{%s}CommunicationMediaType' % self._namespace] = self.communication_media_type
        if self.contact_number_elements:
            for contact_number_element in self.contact_number_elements:
                return_obj.append(contact_number_element.to_obj(ns_info=ns_info))

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.communication_media_type = obj.get('{%s}CommunicationMediaType' % cls._namespace)

        contact_number_elements = obj.findall("{%s}ContactNumberElement" % XML_NS_XPIL)
        if contact_number_elements is not None and len(contact_number_elements) > 0:
            return_obj.contact_number_elements = [ContactNumberElement.from_obj(x) for x in contact_number_elements]

        return return_obj

    def to_dict(self):
        d = {}
        if self.communication_media_type:
            d['communication_media_type'] = self.communication_media_type
        if self.contact_number_elements:
            d['contact_number_elements'] = [x.to_dict() for x in self.contact_number_elements]

        return d

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.communication_media_type = d.get('communication_media_type')
        return_obj.contact_number_elements = [ContactNumberElement.from_dict(x) for x in d.get('contact_number_elements', [])]

        return return_obj


class ContactNumberElement(stix.Entity):
    _namespace = XML_NS_XPIL
    XML_TAG = "{%s}ContactNumberElement" % _namespace

    TYPE_COUNTRY_CODE = "CountryCode"
    TYPE_AREA_CODE = "AreaCode"
    TYPE_LOCAL_NUMBER = "LocalNumber"
    TYPE_EXTENSION = "Extension"
    TYPE_PIN = "Pin"
    TYPE_SEPARATOR = "Separator"
    TYPE_NATIONAL_NUMBER = "NationalNumber"
    TYPE_INTERNATIONAL_NUMBER = "InternationalNumber"

    ALLOWED_TYPES = (
        TYPE_AREA_CODE, TYPE_COUNTRY_CODE, TYPE_EXTENSION,
        TYPE_INTERNATIONAL_NUMBER, TYPE_LOCAL_NUMBER, TYPE_NATIONAL_NUMBER,
        TYPE_SEPARATOR, TYPE_PIN
    )

    def __init__(self, value=None, type_=None):
        self.value = value
        self.type_ = type_

    @property
    def type_(self):
        return self._type

    @type_.setter
    def type_(self, value):
        if not value:
            self._type = None
        elif value not in self.ALLOWED_TYPES:
            raise ValueError('value must be one of %s' % (self.ALLOWED_TYPES,))
        else:
            self._type = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(ContactNumberElement, self).to_obj(ns_info=ns_info)

        return_obj = et.Element(self.XML_TAG)
        if self.type_:
            return_obj.attrib['{%s}Type' % self._namespace] = self.type_
        if self.value:
            return_obj.text = self.value

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if obj is None:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.type_ = obj.get('{%s}Type' % cls._namespace)
        return_obj.value = obj.text
        return return_obj

    def to_dict(self):
        d = {}
        if self.type_:
            d['type'] = self.type_
        if self.value:
            d['value'] = self.value
        return d

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.type_ = d.get('type')
        return_obj.value = d.get('value')
        return return_obj



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest

from stix.core import STIXPackage

from stix.test import EntityTestCase, TypedListTestCase, assert_warnings
from stix.test import data_marking_test
from stix.test.common import (
    confidence_test, information_source_test, related_test,identity_test
)

import stix.threat_actor as ta


class TypesTests(TypedListTestCase, unittest.TestCase):
    klass = ta.ThreatActor

    _partial_dict = [
        {
            'value': {
                "value" : "Hacker",
                "xsi:type" : "stixVocabs:ThreatActorTypeVocab-1.0"
            }
        },
    ]

    _full_dict = {
          "types": _partial_dict
    }

class MotivationsTests(TypedListTestCase, unittest.TestCase):
    klass = ta.ThreatActor

    _partial_dict = [{
             'value': {
                 "value" : "Ego",
                 "xsi:type" : "stixVocabs:MotivationVocab-1.1"
             }
         }]

    _full_dict = {
          "motivations": _partial_dict
    }


class SophisticationTests(TypedListTestCase, unittest.TestCase):
    klass = ta.ThreatActor

    _partial_dict = [
         {
          'value': {
                 "value" : "Novice",
                 "xsi:type" : "stixVocabs:ThreatActorSophisticationVocab-1.0"
         }
         }
    ]

    _full_dict = {
        "sophistications": _partial_dict
    }
    
class IntendedEffectsTests(TypedListTestCase, unittest.TestCase):
    klass = ta.ThreatActor

    _partial_dict = [
        {
         'value': {
                "value" : "Destruction",
                "xsi:type" : "stixVocabs:IntendedEffectVocab-1.0"
         }
        }
    ]
    
    _full_dict = {
        "intended_effects": _partial_dict
    }


class PlanningAndOperationalSupportTests(TypedListTestCase, unittest.TestCase):
    klass = ta.ThreatActor

    _partial_dict = [
        {
         'value': {
            "value" : "Data Exploitation",
            "xsi:type" : 'stixVocabs:PlanningAndOperationalSupportVocab-1.0.1'
         }
        }
    ]

    _full_dict = {
        "planning_and_operational_supports": _partial_dict
    }


class ObservedTTPsTests(EntityTestCase, unittest.TestCase):
    klass = ta.ObservedTTPs

    _full_dict = {
        'scope': 'inclusive',
        'ttps': [
            related_test.RelatedTTPTests._full_dict
        ]
    }


class AssocaitedCampaignsTests(EntityTestCase, unittest.TestCase):
    klass = ta.AssociatedCampaigns

    _full_dict = {
        'scope': 'inclusive',
        'campaigns': [
            related_test.RelatedCampaignTests._full_dict
        ]
    }


class AssociatedActorsTests(EntityTestCase, unittest.TestCase):
    klass = ta.AssociatedActors

    _full_dict = {
        'scope': 'inclusive',
        'threat_actors': [
            related_test.RelatedThreatActorTests._full_dict
        ]
    }


class ThreatActorTests(EntityTestCase, unittest.TestCase):
    klass = ta.ThreatActor
    _full_dict = {
        'id': 'example:ThreatActor-1',
        'timestamp': "2014-01-31T06:14:46",
        'version': '1.1',
        'title': "BadGuy1",
        'description': "This is a long description about a threat actor.",
        'short_description': "A bad guy",
        'identity': identity_test.IdentityTests._full_dict,
        'types': TypesTests._partial_dict,
        'motivations': MotivationsTests._partial_dict,
        'sophistications': SophisticationTests._partial_dict,
        'intended_effects': IntendedEffectsTests._partial_dict,
        'planning_and_operational_supports': PlanningAndOperationalSupportTests._partial_dict,
        'observed_ttps': ObservedTTPsTests._full_dict,
        'associated_campaigns': AssocaitedCampaignsTests._full_dict,
        'associated_actors': AssociatedActorsTests._full_dict,
        'handling': data_marking_test.MarkingTests._full_dict,
        'confidence': confidence_test.ConfidenceTests._full_dict,
        'information_source': information_source_test.InformationSourceTests._full_dict,
        'related_packages': related_test.RelatedPackageRefsTests._full_dict
    }

    def test_add_description(self):
        o1 = self.klass()
        o2 = self.klass()

        o1.add_description("Test")
        o2.descriptions.add("Test")

        self.assertEqual(
            o1.descriptions.to_dict(),
            o2.descriptions.to_dict()
        )

    def test_add_short_description(self):
        o1 = self.klass()
        o2 = self.klass()

        o1.add_short_description("Test")
        o2.short_descriptions.add("Test")

        self.assertEqual(
            o1.short_descriptions.to_dict(),
            o2.short_descriptions.to_dict()
        )

    @assert_warnings
    def test_deprecated_related_packages(self):
        t = ta.ThreatActor()
        t.related_packages.append(STIXPackage())
        self.assertEqual(len(t.related_packages), 1)

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Generated Thu Apr 11 15:07:59 2013 by generateDS.py version 2.9a.
#

import sys

from mixbox.binding_utils import *

from stix.bindings import register_extension
import stix.bindings.exploit_target as exploit_target_binding

XML_NS = "http://stix.mitre.org/extensions/Vulnerability#CVRF-1"

#
# Data representation classes.
#

@register_extension
class CVRF1_1InstanceType(exploit_target_binding.VulnerabilityType):
    """The CVRF1.1InstanceType provides an extension to the
    exploit_target_binding.VulnerabilityType which imports and leverages the CVRF schema
    for structured characterization of Vulnerabilities. This could
    include characterization of 0-days or other vulnerabilities that
    do not have a CVE or OSVDB ID."""
    subclass = None
    superclass = exploit_target_binding.VulnerabilityType

    xmlns          = XML_NS
    xmlns_prefix   = "cvrfVuln"
    xml_type       = "CVRF1.1InstanceType"
    xsi_type       = "%s:%s" % (xmlns_prefix, xml_type)

    def __init__(self, Description=None, CVE_ID=None, OSVDB_ID=None, CVSS_Score=None, cvrfdoc=None):
        super(CVRF1_1InstanceType, self).__init__(Description=Description, CVE_ID=CVE_ID, OSVDB_ID=OSVDB_ID, CVSS_Score=CVSS_Score)
        self.cvrfdoc = cvrfdoc
    def factory(*args_, **kwargs_):
        if CVRF1_1InstanceType.subclass:
            return CVRF1_1InstanceType.subclass(*args_, **kwargs_)
        else:
            return CVRF1_1InstanceType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_cvrfdoc(self): return self.cvrfdoc
    def set_cvrfdoc(self, cvrfdoc): self.cvrfdoc = cvrfdoc
    def hasContent_(self):
        if (
            self.cvrfdoc is not None or
            super(CVRF1_1InstanceType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, nsmap, namespace_=XML_NS, name_='CVRF1.1InstanceType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s:%s%s' % (nsmap[namespace_], name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='CVRF1.1InstanceType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, nsmap, XML_NS, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s:%s>%s' % (nsmap[namespace_], name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='', name_='CVRF1.1InstanceType'):
        super(CVRF1_1InstanceType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='CVRF1.1InstanceType')
        if 'xmlns' not in already_processed:
            already_processed.add('xmlns')
            xmlns = " xmlns:%s='%s'" % (self.xmlns_prefix, self.xmlns)
            lwrite(xmlns)
        if 'xsi:type' not in already_processed:
            already_processed.add('xsi:type')
            xsi_type = " xsi:type='%s:%s'" % (self.xmlns_prefix, self.xml_type)
            lwrite(xsi_type)
    def exportChildren(self, lwrite, level, nsmap, namespace_=XML_NS, name_='CVRF1.1InstanceType', fromsubclass_=False, pretty_print=True):
        super(CVRF1_1InstanceType, self).exportChildren(lwrite, level, nsmap, namespace_, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.cvrfdoc is not None:
            showIndent(lwrite, level, pretty_print)
            lwrite(etree_.tostring(self.cvrfdoc, pretty_print=pretty_print).decode())
            #self.cvrfdoc.export(lwrite, level, nsmap, namespace_, name_='cvrfdoc', pretty_print=pretty_print)
    def build(self, node):
        self.__sourcenode__ = node
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(CVRF1_1InstanceType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'cvrfdoc':
            self.set_cvrfdoc(child_)
        super(CVRF1_1InstanceType, self).buildChildren(child_, node, nodeName_, True)
# end class CVRF1_1InstanceType

GDSClassesMapping = {}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print(USAGE_TEXT)
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'CVRF1.1InstanceType'
        rootClass = CVRF1_1InstanceType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('<?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout, 0, name_=rootTag,
        namespacedef_='',
        pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'CVRF1.1InstanceType'
        rootClass = CVRF1_1InstanceType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from mixbox.vendor.six import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'CVRF1.1InstanceType'
        rootClass = CVRF1_1InstanceType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    sys.stdout.write('<?xml version="1.0" ?>\n')
    rootObj.export(sys.stdout, 0, name_="CVRF1.1InstanceType",
        namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "CVRF1_1InstanceType"
    ]



#!/usr/bin/env python
# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

"""
Description: Round-trip example. This script takes a STIX instance document from XML to
a binding object, then to a api object and then to a dictionary. That dictionary is then
converted back into an api object, which is then used to generate an XML document.
"""
# stdlib
from pprint import pprint

# python-stix
from stix.core import STIXPackage


def main():
    FILENAME = 'sample.xml'

    # Parse input file
    stix_package = STIXPackage.from_xml(FILENAME)

    # Convert STIXPackage to a Python dictionary via the to_dict() method.
    stix_dict = stix_package.to_dict()

    # Print the dictionary!
    pprint(stix_dict)

    # Convert the first STIXPackage dictionary into another STIXPackage via
    # the from_dict() method.
    stix_package_two = STIXPackage.from_dict(stix_dict)

    # Serialize the new STIXPackage object to XML
    xml = stix_package_two.to_xml()

    # Print the XML!
    print(xml)

if __name__ == '__main__':
    main()



# Copyright (c) 2015, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

# internal
import stix
import stix.utils as utils
import stix.bindings.incident as incident_binding
from stix.common import DateTimeWithPrecision
from stix.coa import CourseOfAction

# relative
from .contributors import Contributors

from mixbox import entities, fields

class COATaken(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = _binding.COATakenType
    
    course_of_action = fields.TypedField("Course_Of_Action", CourseOfAction)
    contributors = fields.TypedField("Contributors", Contributors)
    time = fields.TypedField("Time", type_="stix.incident.coa.COATime")
    
    def __init__(self, course_of_action=None):
        super(COATaken, self).__init__()
        self.time = None
        self.course_of_action = course_of_action

    def add_contributor(self, value):
        self.contributors.append(value)


class COARequested(COATaken):
    namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = _binding.COARequestedType

    priority = fields.TypedField("priority")

    def __init__(self, course_of_action=None):
        super(COARequested, self).__init__(course_of_action=course_of_action)


class COATime(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = _binding.COATimeType
    
    start = fields.TypedField("Start", DateTimeWithPrecision)
    end = fields.TypedField("End", DateTimeWithPrecision)
    
    def __init__(self, start=None, end=None):
        super(COATime, self).__init__()
        self.start = start
        self.end = end
